@@ -2,8 +2,8 @@ using System;
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
-[assembly: ApplicationVersion ("1.5.1", "1.6 Beta 2")]
-[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "x86_64", "2009-10-12 14:02:32 EDT")]
+[assembly: ApplicationVersion ("1.5.2", "1.6 Beta 3")]
+[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "i686", "2009-11-20 19:17:59 EST")]
 
 [assembly: AssemblyVersion ("1.5.0.0")]
 [assembly: AssemblyTitle ("Banshee")]
