@@ -5,31 +5,31 @@
  *  Written by Aaron Bockover <aaron@abock.org>
  ****************************************************************************/
 
-/*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW: 
+/*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW:
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
- *  copy of this software and associated documentation files (the "Software"),  
- *  to deal in the Software without restriction, including without limitation  
- *  the rights to use, copy, modify, merge, publish, distribute, sublicense,  
- *  and/or sell copies of the Software, and to permit persons to whom the  
+ *  copy of this software and associated documentation files (the "Software"),
+ *  to deal in the Software without restriction, including without limitation
+ *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ *  and/or sell copies of the Software, and to permit persons to whom the
  *  Software is furnished to do so, subject to the following conditions:
  *
- *  The above copyright notice and this permission notice shall be included in 
+ *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
- *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
- *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
- *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
- *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  *  DEALINGS IN THE SOFTWARE.
  */
- 
+
 using System;
 using System.Runtime.InteropServices;
 
-namespace Banshee.Base 
+namespace Banshee.Base
 {
     public class SafeUri
     {
@@ -38,14 +38,14 @@ namespace Banshee.Base
             Yes,
             No
         }
-        
+
         private static int MAX_SCHEME_LENGTH = 6;
 
         private string uri;
         private string local_path;
         private string scheme;
         private LocalPathCheck local_path_check = LocalPathCheck.NotPerformed;
-        
+
         public SafeUri (string uri)
         {
             int scheme_delimit_index = uri.IndexOf ("://");
@@ -55,7 +55,7 @@ namespace Banshee.Base
                 this.uri = FilenameToUri (uri);
             }
         }
-        
+
         public SafeUri (string uri, bool isUri)
         {
             if (isUri) {
@@ -64,7 +64,7 @@ namespace Banshee.Base
                 this.uri = FilenameToUri (uri);
             }
         }
-        
+
         public SafeUri (Uri uri)
         {
             this.uri = uri.AbsoluteUri;
@@ -83,46 +83,46 @@ namespace Banshee.Base
 
             string uri = GLib.Marshaller.Utf8PtrToString (uri_ptr);
             GLib.Marshaller.Free (uri_ptr);
-            
+
             return uri;
         }
-        
+
         public static string UriToFilename (string uri)
         {
             // TODO: replace with managed conversion to avoid marshalling
             IntPtr uri_ptr = GLib.Marshaller.StringToPtrGStrdup (uri);
             IntPtr path_ptr = g_filename_from_uri (uri_ptr, IntPtr.Zero, IntPtr.Zero);
             GLib.Marshaller.Free (uri_ptr);
-            
+
             if (path_ptr == IntPtr.Zero) {
                 throw new ApplicationException ("URI could not be converted to local file location");
             }
-            
+
             string path = GLib.Marshaller.Utf8PtrToString (path_ptr);
             GLib.Marshaller.Free (path_ptr);
-            
+
             return path;
         }
-        
+
         public static string UriToFilename (SafeUri uri)
         {
             return UriToFilename (uri.AbsoluteUri);
         }
-        
+
         public override string ToString ()
         {
             return AbsoluteUri;
         }
-        
+
         public override bool Equals (object o)
         {
             if (!(o is SafeUri)) {
                 return false;
             }
-            
+
             return (o as SafeUri).AbsoluteUri == AbsoluteUri;
         }
-        
+
         public override int GetHashCode ()
         {
             return AbsoluteUri.GetHashCode ();
@@ -131,7 +131,7 @@ namespace Banshee.Base
         public string AbsoluteUri {
             get { return uri; }
         }
-        
+
         public bool IsLocalPath {
             get {
                 if (local_path_check == LocalPathCheck.NotPerformed) {
@@ -143,43 +143,43 @@ namespace Banshee.Base
                         return false;
                     }
                 }
-                
+
                 return local_path_check == LocalPathCheck.Yes;
             }
-        }               
-        
+        }
+
         public string AbsolutePath {
             get {
                 if (local_path == null && IsLocalPath) {
                     local_path = UriToFilename (uri);
                 }
-                
+
                 return local_path;
             }
         }
-        
+
         public string LocalPath {
             get { return AbsolutePath; }
         }
-        
+
         public string Scheme {
             get {
                 if (scheme == null) {
                     scheme = uri.Substring (0, uri.IndexOf ("://"));
                 }
-                
+
                 return scheme;
             }
         }
-        
+
         public bool IsFile {
             get { return Scheme == System.Uri.UriSchemeFile; }
         }
-        
-        [DllImport ("libglib-2.0.so.0")]
+
+        [DllImport ("libglib-2.0-0.dll")]
         private static extern IntPtr g_filename_to_uri (IntPtr filename, IntPtr hostname, IntPtr error);
 
-        [DllImport ("libglib-2.0.so.0")]
+        [DllImport ("libglib-2.0-0.dll")]
         private static extern IntPtr g_filename_from_uri (IntPtr uri, IntPtr hostname, IntPtr error);
     }
 }
