@@ -40,15 +40,15 @@ namespace Banshee.Streaming
         // This is a short list of video types that TagLib# might not support but that
         // we want to make sure are recognized as videos
         private static readonly ExtensionSet VideoExtensions = new ExtensionSet (
-            "avi", "divx", "dv", "f4p", "f4v", "flv", "m4v", "mkv", "mov", "ogv", "qt");
-        
+            "avi", "divx", "dv", "f4p", "f4v", "flv", "m4v", "mkv", "mov", "ogv", "qt", "ts");
+
         public static TagLib.File ProcessUri (SafeUri uri)
         {
             try {
-                TagLib.File file = Banshee.IO.DemuxVfs.OpenFile (uri.IsLocalPath ? uri.LocalPath : uri.AbsoluteUri, 
+                TagLib.File file = Banshee.IO.DemuxVfs.OpenFile (uri.IsLocalPath ? uri.LocalPath : uri.AbsoluteUri,
                     null, TagLib.ReadStyle.Average);
-    
-                if ((file.Properties.MediaTypes & TagLib.MediaTypes.Audio) == 0 && 
+
+                if ((file.Properties.MediaTypes & TagLib.MediaTypes.Audio) == 0 &&
                     (file.Properties.MediaTypes & TagLib.MediaTypes.Video) == 0) {
                     throw new TagLib.UnsupportedFormatException ("File does not contain video or audio");
                 }
@@ -63,40 +63,45 @@ namespace Banshee.Streaming
         {
             return Choose (priority, fallback, false);
         }
-    
+
         private static string Choose (string priority, string fallback, bool flip)
         {
-            return flip 
-                ? String.IsNullOrEmpty (fallback) ? priority : fallback
-                : String.IsNullOrEmpty (priority) ? fallback : priority;
+            return flip
+                ? IsNullOrEmpty (fallback) ? priority : fallback
+                : IsNullOrEmpty (priority) ? fallback : priority;
+        }
+
+        private static bool IsNullOrEmpty (string s)
+        {
+            return String.IsNullOrEmpty (s) || s.IndexOf ('\0') >= 0 && s.Trim ('\0').Length == 0;
         }
 
         #pragma warning disable 0169
-        
+
         private static int Choose (int priority, int fallback)
         {
             return Choose (priority, fallback, false);
         }
-        
+
         #pragma warning restore 0169
-        
+
         private static int Choose (int priority, int fallback, bool flip)
         {
-            return flip 
+            return flip
                 ? (fallback <= 0 ? priority : fallback)
                 : (priority <= 0 ? fallback : priority);
         }
-        
+
         private static void FindTrackMediaAttributes (TrackInfo track, TagLib.File file)
         {
             if ((file.Properties.MediaTypes & TagLib.MediaTypes.Audio) != 0) {
                 track.MediaAttributes |= TrackMediaAttributes.AudioStream;
             }
-            
+
             if ((file.Properties.MediaTypes & TagLib.MediaTypes.Video) != 0) {
                 track.MediaAttributes |= TrackMediaAttributes.VideoStream;
             }
-            
+
             if (file.Tag.FirstGenre == "Podcast" || file.Tag.Album == "Podcast") {
                 track.MediaAttributes |= TrackMediaAttributes.Podcast;
             }
@@ -121,13 +126,18 @@ namespace Banshee.Streaming
                 track.TrackTitle = uri.AbsoluteUri;
             }
         }
-        
+
         public static void TrackInfoMerge (TrackInfo track, TagLib.File file)
         {
             TrackInfoMerge (track, file, false);
         }
 
         public static void TrackInfoMerge (TrackInfo track, TagLib.File file, bool preferTrackInfo)
+        {
+            TrackInfoMerge (track, file, preferTrackInfo, false);
+        }
+
+        public static void TrackInfoMerge (TrackInfo track, TagLib.File file, bool preferTrackInfo, bool import_rating_and_play_count)
         {
             // TODO support these as arrays:
             // Performers[] (track artists), AlbumArtists[], Composers[], Genres[]
@@ -139,9 +149,9 @@ namespace Banshee.Streaming
                 track.MimeType = file.MimeType;
                 track.Duration = file.Properties.Duration;
                 track.BitRate  = file.Properties.AudioBitrate;
-                
+
                 FindTrackMediaAttributes (track, file);
-    
+
                 track.ArtistName = Choose (file.Tag.JoinedPerformers, track.ArtistName, preferTrackInfo);
                 track.ArtistNameSort = Choose (file.Tag.JoinedPerformersSort, track.ArtistNameSort, preferTrackInfo);
                 track.AlbumTitle = Choose (file.Tag.Album, track.AlbumTitle, preferTrackInfo);
@@ -149,7 +159,7 @@ namespace Banshee.Streaming
                 track.AlbumArtist = Choose (file.Tag.FirstAlbumArtist, track.AlbumArtist, preferTrackInfo);
                 track.AlbumArtistSort = Choose (file.Tag.FirstAlbumArtistSort, track.AlbumArtistSort, preferTrackInfo);
                 track.IsCompilation = IsCompilation (file);
-                
+
                 track.TrackTitle = Choose (file.Tag.Title, track.TrackTitle, preferTrackInfo);
                 track.TrackTitleSort = Choose (file.Tag.TitleSort, track.TrackTitleSort, preferTrackInfo);
                 track.Genre = Choose (file.Tag.FirstGenre, track.Genre, preferTrackInfo);
@@ -158,13 +168,20 @@ namespace Banshee.Streaming
                 track.Grouping = Choose (file.Tag.Grouping, track.Grouping, preferTrackInfo);
                 track.Copyright = Choose (file.Tag.Copyright, track.Copyright, preferTrackInfo);
                 track.Comment = Choose (file.Tag.Comment, track.Comment, preferTrackInfo);
-    
+
                 track.TrackNumber = Choose ((int)file.Tag.Track, track.TrackNumber, preferTrackInfo);
                 track.TrackCount = Choose ((int)file.Tag.TrackCount, track.TrackCount, preferTrackInfo);
                 track.DiscNumber = Choose ((int)file.Tag.Disc, track.DiscNumber, preferTrackInfo);
                 track.DiscCount = Choose ((int)file.Tag.DiscCount, track.DiscCount, preferTrackInfo);
                 track.Year = Choose ((int)file.Tag.Year, track.Year, preferTrackInfo);
                 track.Bpm = Choose ((int)file.Tag.BeatsPerMinute, track.Bpm, preferTrackInfo);
+
+                if (import_rating_and_play_count) {
+                    int file_rating = 0, file_playcount = 0;
+                    StreamRatingTagger.GetRatingAndPlayCount (file, ref file_rating, ref file_playcount);
+                    track.Rating = Choose (file_rating, track.Rating, preferTrackInfo);
+                    track.PlayCount = Choose (file_playcount, track.PlayCount, preferTrackInfo);
+                }
             } else {
                 track.MediaAttributes = TrackMediaAttributes.AudioStream;
                 if (track.Uri != null && VideoExtensions.IsMatchingFile (track.Uri.LocalPath)) {
@@ -184,12 +201,12 @@ namespace Banshee.Streaming
                     }
                 } catch {}
             }
-            
+
             // TODO look for track number in the file name if not set?
             // TODO could also pull artist/album from folders _iff_ files two levels deep in the MusicLibrary folder
             // TODO these ideas could also be done in an extension that collects such hacks
         }
-            
+
         private static bool IsCompilation (TagLib.File file)
         {
             try {
@@ -203,7 +220,7 @@ namespace Banshee.Streaming
                 if (apple_tag != null && apple_tag.IsCompilation)
                     return true;
             } catch {}
-            
+
             // FIXME the FirstAlbumArtist != FirstPerformer check might return true for half the
             // tracks on a compilation album, but false for some
             // TODO checked for 'Soundtrack' (and translated) in the title?
@@ -234,7 +251,7 @@ namespace Banshee.Streaming
             } catch {}
         }
 
-        public static bool SaveToFile (TrackInfo track)
+        public static bool SaveToFile (TrackInfo track, bool write_metadata, bool write_rating_and_play_count)
         {
             // FIXME taglib# does not seem to handle writing metadata to video files well at all atm
             // so not allowing
@@ -242,38 +259,46 @@ namespace Banshee.Streaming
                 Hyena.Log.DebugFormat ("Avoiding 100% cpu bug with taglib# by not writing metadata to video file {0}", track);
                 return false;
             }
-        
+
             // Note: this should be kept in sync with the metadata read in StreamTagger.cs
             TagLib.File file = ProcessUri (track.Uri);
             if (file == null) {
                 return false;
             }
-            
-            file.Tag.Performers = new string [] { track.ArtistName };
-            file.Tag.PerformersSort = new string [] { track.ArtistNameSort };
-            file.Tag.Album = track.AlbumTitle;
-            file.Tag.AlbumSort = track.AlbumTitleSort;
-            file.Tag.AlbumArtists = track.AlbumArtist == null ? new string [0] : new string [] {track.AlbumArtist};
-            file.Tag.AlbumArtistsSort = (track.AlbumArtistSort == null ? new string [0] : new string [] {track.AlbumArtistSort});
-            // Bug in taglib-sharp-2.0.3.0: Crash if you send it a genre of "{ null }"
-            // on a song with both ID3v1 and ID3v2 metadata. It's happy with "{}", though.
-            // (see http://forum.taglib-sharp.com/viewtopic.php?f=5&t=239 )
-            file.Tag.Genres = (track.Genre == null) ? new string[] {} : new string [] { track.Genre };
-            file.Tag.Title = track.TrackTitle;
-            file.Tag.TitleSort = track.TrackTitleSort;
-            file.Tag.Track = (uint)track.TrackNumber;
-            file.Tag.TrackCount = (uint)track.TrackCount;
-            file.Tag.Composers = new string [] { track.Composer };
-            file.Tag.Conductor = track.Conductor;
-            file.Tag.Grouping = track.Grouping;
-            file.Tag.Copyright = track.Copyright;
-            file.Tag.Comment = track.Comment;
-            file.Tag.Disc = (uint)track.DiscNumber;
-            file.Tag.DiscCount = (uint)track.DiscCount;
-            file.Tag.Year = (uint)track.Year;
-            file.Tag.BeatsPerMinute = (uint)track.Bpm;
-            
-            SaveIsCompilation (file, track.IsCompilation);
+
+            if (write_metadata) {
+                file.Tag.Performers = new string [] { track.ArtistName };
+                file.Tag.PerformersSort = new string [] { track.ArtistNameSort };
+                file.Tag.Album = track.AlbumTitle;
+                file.Tag.AlbumSort = track.AlbumTitleSort;
+                file.Tag.AlbumArtists = track.AlbumArtist == null ? new string [0] : new string [] {track.AlbumArtist};
+                file.Tag.AlbumArtistsSort = (track.AlbumArtistSort == null ? new string [0] : new string [] {track.AlbumArtistSort});
+                // Bug in taglib-sharp-2.0.3.0: Crash if you send it a genre of "{ null }"
+                // on a song with both ID3v1 and ID3v2 metadata. It's happy with "{}", though.
+                // (see http://forum.taglib-sharp.com/viewtopic.php?f=5&t=239 )
+                file.Tag.Genres = (track.Genre == null) ? new string[] {} : new string [] { track.Genre };
+                file.Tag.Title = track.TrackTitle;
+                file.Tag.TitleSort = track.TrackTitleSort;
+                file.Tag.Track = (uint)track.TrackNumber;
+                file.Tag.TrackCount = (uint)track.TrackCount;
+                file.Tag.Composers = new string [] { track.Composer };
+                file.Tag.Conductor = track.Conductor;
+                file.Tag.Grouping = track.Grouping;
+                file.Tag.Copyright = track.Copyright;
+                file.Tag.Comment = track.Comment;
+                file.Tag.Disc = (uint)track.DiscNumber;
+                file.Tag.DiscCount = (uint)track.DiscCount;
+                file.Tag.Year = (uint)track.Year;
+                file.Tag.BeatsPerMinute = (uint)track.Bpm;
+
+                SaveIsCompilation (file, track.IsCompilation);
+            }
+
+            if (write_rating_and_play_count) {
+                // FIXME move StreamRatingTagger to taglib#
+                StreamRatingTagger.StoreRatingAndPlayCount (track.Rating, track.PlayCount, file);
+            }
+
             file.Save ();
 
             track.FileSize = Banshee.IO.File.GetSize (track.Uri);
@@ -281,7 +306,7 @@ namespace Banshee.Streaming
             track.LastSyncedStamp = DateTime.Now;
             return true;
         }
-    
+
         public static void TrackInfoMerge (TrackInfo track, StreamTag tag)
         {
             try {
@@ -298,7 +323,7 @@ namespace Banshee.Streaming
                         string title = Choose ((string)tag.Value, track.TrackTitle);
 
                         // Try our best to figure out common patterns in poor radio metadata.
-                        // Often only one tag is sent on track changes inside the stream, 
+                        // Often only one tag is sent on track changes inside the stream,
                         // which is title, and usually contains artist and track title, separated
                         // with a " - " string.
                         if (track.IsLive && title.Contains (" - ")) {
@@ -370,7 +395,7 @@ namespace Banshee.Streaming
                     case CommonTags.MoreInfoUri:
                         track.MoreInfoUri = (SafeUri)tag.Value;
                         break;
-                    /* No year tag in GST it seems 
+                    /* No year tag in GST it seems
                     case CommonTags.Year:
                         track.Year = (uint)tag.Value;
                         break;*/
