@@ -32,15 +32,51 @@ using System.Collections.Generic;
 
 using Mono.Unix;
 
+using Banshee.Collection;
 using Banshee.SmartPlaylist;
+using Banshee.Preferences;
+using Banshee.Configuration;
+using Banshee.Configuration.Schema;
 
 namespace Banshee.Library
 {
     public class MusicLibrarySource : LibrarySource
     {
-        public MusicLibrarySource () : base (Catalog.GetString ("Music Library"), "Library", 40)
+        // Catalog.GetString ("Music Library")
+        public MusicLibrarySource () : base (Catalog.GetString ("Music"), "Library", 40)
         {
+            MediaTypes = TrackMediaAttributes.Music | TrackMediaAttributes.AudioStream;
+            NotMediaTypes = TrackMediaAttributes.Podcast | TrackMediaAttributes.VideoStream | TrackMediaAttributes.AudioBook;
             Properties.SetStringList ("Icon.Name", "audio-x-generic", "source-library");
+
+            // Migrate the old library-location schema, if necessary
+            if (DatabaseConfigurationClient.Client.Get<int> ("MusicLibraryLocationMigrated", 0) != 1) {
+                string old_location = OldLocationSchema.Get ();
+                if (!String.IsNullOrEmpty (old_location)) {
+                    BaseDirectory = old_location;
+                }
+                DatabaseConfigurationClient.Client.Set<int> ("MusicLibraryLocationMigrated", 1);
+            }
+
+            Section file_system = PreferencesPage.Add (new Section ("file-system",
+                Catalog.GetString ("File System Organization"), 5));
+
+            file_system.Add (new SchemaPreference<string> (LibrarySchema.FolderPattern,
+                Catalog.GetString ("Folder hie_rarchy")));
+
+            file_system.Add (new SchemaPreference<string> (LibrarySchema.FilePattern,
+                Catalog.GetString ("File _name")));
+
+            PreferencesPage.Add (new Section ("misc", Catalog.GetString ("Miscellaneous"), 10));
+        }
+
+        public static string GetDefaultBaseDirectory ()
+        {
+            return Banshee.Base.Paths.GetXdgDirectoryUnderHome ("XDG_MUSIC_DIR", "Music");
+        }
+
+        public override string DefaultBaseDirectory {
+            get { return GetDefaultBaseDirectory (); }
         }
 
         public override IEnumerable<SmartPlaylistDefinition> DefaultSmartPlaylists {
@@ -76,7 +112,7 @@ namespace Banshee.Library
         private static SmartPlaylistDefinition [] non_default_smart_playlists = new SmartPlaylistDefinition [] {
             new SmartPlaylistDefinition (
                 Catalog.GetString ("Neglected Favorites"),
-                Catalog.GetString ("Favorites not played in over two weeks"),
+                Catalog.GetString ("Favorites not played in over two months"),
                 "rating>=4 played>=\"2 months ago\""),
 
             new SmartPlaylistDefinition (
