@@ -66,7 +66,7 @@ namespace Banshee.Gui
             get { return (SourceView.HighlightedSource as PrimarySource) ?? base.ActivePrimarySource; }
         }
 
-        public SourceActions (InterfaceActionService actionService) : base (actionService, "Source")
+        public SourceActions () : base ("Source")
         {
             Add (new ActionEntry [] {
                 new ActionEntry ("NewPlaylistAction", null,
@@ -74,14 +74,14 @@ namespace Banshee.Gui
                     Catalog.GetString ("Create a new empty playlist"), OnNewPlaylist),
 
                 new ActionEntry ("NewSmartPlaylistAction", null,
-                    Catalog.GetString ("New _Smart Playlist"), null,
+                    Catalog.GetString ("New _Smart Playlist..."), null,
                     Catalog.GetString ("Create a new smart playlist"), OnNewSmartPlaylist),
 
                 /*new ActionEntry ("NewSmartPlaylistFromSearchAction", null,
                     Catalog.GetString ("New _Smart Playlist _From Search"), null,
                     Catalog.GetString ("Create a new smart playlist from the current search"), OnNewSmartPlaylistFromSearch),*/
 
-                new ActionEntry ("SourceContextMenuAction", null, 
+                new ActionEntry ("SourceContextMenuAction", null,
                     String.Empty, null, null, OnSourceContextMenu),
 
                 new ActionEntry ("ImportSourceAction", null,
@@ -97,42 +97,32 @@ namespace Banshee.Gui
 
                 new ActionEntry ("UnmapSourceAction", null,
                     Catalog.GetString ("Unmap"), "<shift>Delete", null, OnUnmapSource),
-                    
+
                 new ActionEntry ("SourcePropertiesAction", null,
                     Catalog.GetString ("Source Properties"), null, null, OnSourceProperties),
+
+                new ActionEntry ("SortChildrenAction", Stock.SortDescending,
+                    Catalog.GetString ("Sort Children by"), null, null,
+                    OnSortChildrenMenu),
+
+                new ActionEntry ("SourcePreferencesAction", null, Catalog.GetString ("Preferences"), null,
+                    Catalog.GetString ("Edit preferences related to this source"), OnSourcePreferences),
+
             });
 
+            this["NewSmartPlaylistAction"].ShortLabel = Catalog.GetString ("New _Smart Playlist");
             this["NewPlaylistAction"].IconName = Stock.New;
             this["UnmapSourceAction"].IconName = Stock.Delete;
             this["SourcePropertiesAction"].IconName = Stock.Properties;
+            this["SortChildrenAction"].HideIfEmpty = false;
+            this["SourcePreferencesAction"].IconName = Stock.Preferences;
 
             AddImportant (
                 new ActionEntry ("RefreshSmartPlaylistAction", Stock.Refresh,
                     Catalog.GetString ("Refresh"), null,
                     Catalog.GetString ("Refresh this randomly sorted smart playlist"), OnRefreshSmartPlaylist)
             );
-            
-            Add (new Gtk.ActionEntry [] {
-                new Gtk.ActionEntry ("SortChildrenAction", null, 
-                    Catalog.GetString ("Sort Children"), null, null, null),
-                    
-                new Gtk.ActionEntry ("SortChildrenNameAscAction", null, 
-                    Catalog.GetString ("Name Ascending"), null, null, 
-                    delegate { ActionSource.SortChildSources (new Source.NameComparer (), true); }),
-                    
-                new Gtk.ActionEntry ("SortChildrenNameDescAction", null, 
-                    Catalog.GetString ("Name Descending"), null, null, 
-                    delegate { ActionSource.SortChildSources (new Source.NameComparer (), false); }),
-                    
-                new Gtk.ActionEntry ("SortChildrenSizeAscAction", null, 
-                    Catalog.GetString ("Size Ascending"), null, null, 
-                    delegate { ActionSource.SortChildSources (new Source.SizeComparer (), true); }),
-                    
-                new Gtk.ActionEntry ("SortChildrenSizeDescAction", null, 
-                    Catalog.GetString ("Size Descending"), null, null, 
-                    delegate { ActionSource.SortChildSources (new Source.SizeComparer (), false); })
-            });
-                
+
             //ServiceManager.SourceManager.SourceUpdated += OnPlayerEngineStateChanged;
             //ServiceManager.SourceManager.SourceViewChanged += OnPlayerEngineStateChanged;
             //ServiceManager.SourceManager.SourceAdded += OnPlayerEngineStateChanged;
@@ -140,18 +130,18 @@ namespace Banshee.Gui
             ServiceManager.SourceManager.ActiveSourceChanged += HandleActiveSourceChanged;
             Actions.GlobalActions["EditMenuAction"].Activated += HandleEditMenuActivated;
         }
-            
+
 #region State Event Handlers
 
         private void HandleActiveSourceChanged (SourceEventArgs args)
         {
             Banshee.Base.ThreadAssist.ProxyToMain (delegate {
                 UpdateActions ();
-                
+
                 if (last_source != null) {
                     last_source.Updated -= HandleActiveSourceUpdated;
                 }
-                
+
                 if (ActiveSource != null) {
                     ActiveSource.Updated += HandleActiveSourceUpdated;
                 }
@@ -162,7 +152,7 @@ namespace Banshee.Gui
         {
             UpdateActions ();
         }
-        
+
         private void HandleActiveSourceUpdated (object o, EventArgs args)
         {
             Banshee.Base.ThreadAssist.ProxyToMain (delegate {
@@ -245,7 +235,7 @@ namespace Banshee.Gui
                 return;
             }
 
-            menu.Show (); 
+            menu.Show ();
             menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
             menu.SelectionDone += delegate {
                 SourceView.ResetHighlight ();
@@ -269,30 +259,30 @@ namespace Banshee.Gui
             if (source == null) {
                 return;
             }
+            source.Activate ();
 
             PlaylistExportDialog chooser = new PlaylistExportDialog (source.Name, PrimaryWindow);
 
             string uri = null;
             PlaylistFormatDescription format = null;
-            int response = chooser.Run ();            
-            if (response == (int) ResponseType.Ok) {                    
+            int response = chooser.Run ();
+            if (response == (int) ResponseType.Ok) {
                 uri = chooser.Uri;
                 // Get the format that the user selected.
                 format = chooser.GetExportFormat ();
-            }             
-            chooser.Destroy (); 
+            }
+            chooser.Destroy ();
 
             if (uri == null) {
                 // User cancelled export.
                 return;
             }
-            
+
             try {
                 IPlaylistFormat playlist = (IPlaylistFormat)Activator.CreateInstance (format.Type);
                 SafeUri suri = new SafeUri (uri);
                 if (suri.IsLocalPath) {
                     playlist.BaseUri = new Uri (System.IO.Path.GetDirectoryName (suri.LocalPath));
-                    Console.WriteLine (playlist.BaseUri.LocalPath);
                 }
                 playlist.Save (Banshee.IO.File.OpenWrite (new SafeUri (uri), true), source);
             } catch (Exception e) {
@@ -331,6 +321,30 @@ namespace Banshee.Gui
             }
         }
 
+        private void OnSortChildrenMenu (object o, EventArgs args)
+        {
+            foreach (Widget proxy_widget in this["SortChildrenAction"].Proxies) {
+                MenuItem menu = proxy_widget as MenuItem;
+                if (menu == null)
+                    continue;
+
+                Menu submenu = BuildSortMenu (ActionSource);
+                menu.Submenu = submenu;
+                submenu.ShowAll ();
+            }
+        }
+
+        private void OnSourcePreferences (object o, EventArgs args)
+        {
+            try {
+                Banshee.Preferences.Gui.PreferenceDialog dialog = new Banshee.Preferences.Gui.PreferenceDialog ();
+                dialog.ShowSourcePageId (ActionSource.PreferencesPageId);
+                dialog.Run ();
+                dialog.Destroy ();
+            } catch (ApplicationException) {
+            }
+        }
+
 #endregion
 
 #region Utility Methods
@@ -340,7 +354,7 @@ namespace Banshee.Gui
         {
             UpdateActions (false);
         }
-        
+
         private void UpdateActions (bool force)
         {
             Source source = ActionSource;
@@ -356,6 +370,7 @@ namespace Banshee.Gui
                 UpdateAction ("ImportSourceAction", import_source != null, import_source != null && import_source.CanImport, source);
                 UpdateAction ("ExportPlaylistAction", source is AbstractPlaylistSource, true, source);
                 UpdateAction ("SourcePropertiesAction", source.HasProperties, true, source);
+                UpdateAction ("SourcePreferencesAction", source.PreferencesPageId != null, true, source);
                 UpdateAction ("RefreshSmartPlaylistAction", smart_playlist != null && smart_playlist.CanRefresh, true, source);
 
                 bool playlists_writable = primary_source != null && primary_source.SupportsPlaylists && !primary_source.PlaylistsReadOnly;
@@ -363,7 +378,7 @@ namespace Banshee.Gui
                 UpdateAction ("NewSmartPlaylistAction", playlists_writable, true, source);
                 /*UpdateAction ("NewSmartPlaylistFromSearchAction", (source is LibrarySource || source.Parent is LibrarySource),
                         !String.IsNullOrEmpty (source.FilterQuery), source);*/
-                    
+
                 ActionGroup browser_actions = Actions.FindActionGroup ("BrowserView");
                 if (browser_actions != null) {
                     IFilterableSource filterable_source = source as IFilterableSource;
@@ -375,9 +390,9 @@ namespace Banshee.Gui
 
                 last_source = source;
             }
-            
+
             if (source != null) {
-                UpdateAction ("SortChildrenAction", source.Children.Count > 1, true, null);
+                UpdateAction ("SortChildrenAction", source.ChildSortTypes.Length > 0 && source.Children.Count > 1, true, source);
             }
 
             Action<Source> handler = Updated;
@@ -390,11 +405,11 @@ namespace Banshee.Gui
         {
             string key = "no_confirm_unmap_" + source.GetType ().Name.ToLower ();
             bool do_not_ask = ConfigurationClient.Get<bool> ("sources", key, false);
-            
+
             if (do_not_ask) {
                 return true;
             }
-        
+
             Banshee.Widgets.HigMessageDialog dialog = new Banshee.Widgets.HigMessageDialog (
                 ServiceManager.Get<GtkElementsService> ("GtkElementsService").PrimaryWindow,
                 Gtk.DialogFlags.Modal,
@@ -403,9 +418,9 @@ namespace Banshee.Gui
                 String.Format (Catalog.GetString ("Are you sure you want to delete this {0}?"),
                     source.GenericName.ToLower ()),
                 source.Name);
-            
+
             dialog.AddButton (Gtk.Stock.Delete, Gtk.ResponseType.Ok, false);
-            
+
             Gtk.Alignment alignment = new Gtk.Alignment (0.0f, 0.0f, 0.0f, 0.0f);
             alignment.TopPadding = 10;
             Gtk.CheckButton confirm_button = new Gtk.CheckButton (String.Format (Catalog.GetString (
@@ -416,20 +431,60 @@ namespace Banshee.Gui
             alignment.Add (confirm_button);
             alignment.ShowAll ();
             dialog.LabelVBox.PackStart (alignment, false, false, 0);
-            
+
             try {
                 if (dialog.Run () == (int)Gtk.ResponseType.Ok) {
                     ConfigurationClient.Set<bool> ("sources", key, do_not_ask);
                     return true;
                 }
-                
+
                 return false;
             } finally {
                 dialog.Destroy ();
             }
         }
 
+        private Menu BuildSortMenu (Source source)
+        {
+            Menu menu = new Menu ();
+            GLib.SList group = null;
+            foreach (SourceSortType sort_type in source.ChildSortTypes) {
+                RadioMenuItem item = new RadioMenuItem (group, sort_type.Label);
+                group = item.Group;
+                item.Active = (sort_type == source.ActiveChildSort);
+                item.Toggled += BuildSortChangedHandler (source, sort_type);
+                menu.Append (item);
+            }
+
+            menu.Append (new SeparatorMenuItem ());
+
+            CheckMenuItem sort_types_item = new CheckMenuItem (Catalog.GetString ("Separate by Type"));
+            sort_types_item.Active = source.SeparateChildrenByType;
+            sort_types_item.Toggled += OnSeparateTypesChanged;
+            menu.Append (sort_types_item);
+
+            return menu;
+        }
+
+        private void OnSeparateTypesChanged (object o, EventArgs args)
+        {
+            CheckMenuItem item = o as CheckMenuItem;
+            if (item != null) {
+                ActionSource.SeparateChildrenByType = item.Active;
+            }
+        }
+
+        private static EventHandler BuildSortChangedHandler (Source source, SourceSortType sort_type)
+        {
+            return delegate (object sender, EventArgs args) {
+                RadioMenuItem item = sender as RadioMenuItem;
+                if (item != null && item.Active) {
+                    source.SortChildSources (sort_type);
+                }
+            };
+        }
+
 #endregion
-        
+
     }
 }
