@@ -33,8 +33,8 @@ using System.Collections.Generic;
 using Mono.Unix;
 
 using Lastfm;
+using Lastfm.Gui;
 using Hyena.Data;
-using SortType = Hyena.Data.SortType;
 
 using Banshee.Base;
 using Banshee.Collection;
@@ -43,13 +43,14 @@ using Banshee.Sources;
 using Banshee.MediaEngine;
 using Banshee.ServiceStack;
 using Banshee.Networking;
+using Banshee.Preferences;
 
 using Banshee.Sources.Gui;
 
 using Browser = Lastfm.Browser;
 
 namespace Banshee.Lastfm.Radio
-{   
+{
     public class LastfmSource : Source, IDisposable
     {
         private const string lastfm = "Last.fm";
@@ -69,24 +70,24 @@ namespace Banshee.Lastfm.Radio
             get { return actions; }
         }
 
-        public LastfmSource () : base (lastfm, lastfm, 210)
+        public LastfmSource () : base (lastfm, lastfm, 210, lastfm)
         {
-            TypeUniqueId = lastfm;
             account = LastfmCore.Account;
 
             // We don't automatically connect to Last.fm, but load the last Last.fm
             // username we used so we can load the user's stations.
             if (account.UserName != null) {
                 account.UserName = LastUserSchema.Get ();
-                account.CryptedPassword = LastPassSchema.Get ();
+                account.SessionKey = LastSessionKeySchema.Get ();
+                account.Subscriber = LastIsSubscriberSchema.Get ();
             }
 
             if (LastfmCore.UserAgent == null) {
                 LastfmCore.UserAgent = Banshee.Web.Browser.UserAgent;
             }
-            
+
             Browser.Open = Banshee.Web.Browser.Open;
-            
+
             connection = LastfmCore.Radio;
             Network network = ServiceManager.Get<Network> ();
             connection.UpdateNetworkState (network.Connected);
@@ -98,9 +99,10 @@ namespace Banshee.Lastfm.Radio
             UpdateUI ();
 
             Properties.SetString ("ActiveSourceUIResource", "ActiveSourceUI.xml");
+            Properties.Set<bool> ("ActiveSourceUIResourcePropagate", true);
             Properties.SetString ("GtkActionPath", "/LastfmSourcePopup");
             Properties.SetString ("Icon.Name", "lastfm-audioscrobbler");
-            Properties.SetString ("SourcePropertiesActionLabel", Catalog.GetString ("Edit Last.fm Settings"));
+            Properties.SetString ("SortChildrenActionLabel", Catalog.GetString ("Sort Stations by"));
             Properties.Set<LastfmColumnController> ("TrackView.ColumnController", new LastfmColumnController ());
 
             // FIXME this is temporary until we split the GUI part from the non-GUI part
@@ -108,6 +110,7 @@ namespace Banshee.Lastfm.Radio
             Properties.Set<bool> ("Nereid.SourceContents.HeaderVisible", false);
 
             actions = new LastfmActions (this);
+            InstallPreferences ();
 
             ServiceManager.SourceManager.AddSource (this);
 
@@ -119,8 +122,9 @@ namespace Banshee.Lastfm.Radio
             ServiceManager.Get<DBusCommandService> ().ArgumentPushed -= OnCommandLineArgument;
             Connection.StateChanged -= HandleConnectionStateChanged;
             Connection.Dispose ();
+            UninstallPreferences ();
             actions.Dispose ();
-            
+
             actions = null;
             connection = null;
             account = null;
@@ -131,27 +135,13 @@ namespace Banshee.Lastfm.Radio
             if (!isFile || String.IsNullOrEmpty (uri)) {
                 return;
             }
-            
+
             // Handle lastfm:// URIs
             if (uri.StartsWith ("lastfm://")) {
                 StationSource.CreateFromUrl (this, uri);
             }
         }
 
-        /*public override void AddChildSource (ChildSource source)
-        {
-            base.AddChildSource (source);
-            SortChildSources ();
-            source.Updated += HandleChildUpdated;
-        }
-
-        public override void RemoveChildSource (ChildSource source)
-        {
-            base.RemoveChildSource (source);
-            source.Updated -= HandleChildUpdated;
-        }*/
-
-
         // Order by the playCount of a station, then by inverted name
         public class PlayCountComparer : IComparer<Source>
         {
@@ -159,86 +149,40 @@ namespace Banshee.Lastfm.Radio
             {
                 StationSource a = sa as StationSource;
                 StationSource b = sb as StationSource;
-                int c = a.PlayCount.CompareTo (b.PlayCount);
-                return c == 0 ? -(a.Name.CompareTo (b.Name)) : c; 
+                return a.PlayCount.CompareTo (b.PlayCount);
             }
         }
 
-        // Order by the type of station, then by the station name
-        public class TypeComparer : IComparer<Source>
-        {
-            public int Compare (Source sa, Source sb)
-            {
-                StationSource a = sa as StationSource;
-                StationSource b = sb as StationSource;
-                int c = a.Type.Name.CompareTo (b.Type.Name);
-                return c == 0 ? (a.Name.CompareTo (b.Name)) : c; 
-            }
-        }
-
-        public static IComparer<Source> [] ChildComparers = new IComparer<Source> [] {
-            new NameComparer (), new PlayCountComparer (), new TypeComparer ()
-        };
-        public static SortType [] child_orders = new SortType [] {
-            SortType.Ascending, SortType.Descending, SortType.Ascending
+        private static SourceSortType[] sort_types = new SourceSortType[] {
+            SortNameAscending,
+            new SourceSortType (
+                "LastfmTotalPlayCount",
+                Catalog.GetString ("Total Play Count"),
+                SortType.Descending, new PlayCountComparer ())
         };
 
-        public IComparer<Source> ChildComparer {
-            get {
-                if (child_comparer == null) {
-                    int i = (int) StationSortSchema.Get ();
-                    ChildComparer = ChildComparers [i];
-                }
-                return child_comparer;
-            }
-            set {
-                child_comparer = value;
-                int i = Array.IndexOf (ChildComparers, child_comparer);
-                child_order = child_orders[i];
-                StationSortSchema.Set (i);
-            }
+        public override SourceSortType[] ChildSortTypes {
+            get { return sort_types; }
         }
 
-        private bool sorting = false;
-        private IComparer<Source> child_comparer;
-        private SortType child_order;
-        public override void SortChildSources (IComparer<Source> comparer, bool asc)
-        {
-            lock (this) {
-                if (sorting)
-                    return;
-                sorting = true;
-            }
-
-            base.SortChildSources (comparer, asc);
-            ChildComparer = comparer;
-            sorting = false;
-        }
-
-        public void SortChildSources  ()
-        {
-            SortChildSources (ChildComparer, child_order == SortType.Ascending);
+        public override SourceSortType DefaultChildSort {
+            get { return SortNameAscending; }
         }
 
         private string last_username;
         private bool last_was_subscriber = false;
         public void SetUserName (string username)
         {
-            if (username != last_username || last_was_subscriber != Connection.Subscriber) {
+            if (username != last_username || last_was_subscriber != Account.Subscriber) {
                 last_username = username;
-                last_was_subscriber = Connection.Subscriber;
+                last_was_subscriber = Account.Subscriber;
                 LastfmSource.LastUserSchema.Set (last_username);
                 ClearChildSources ();
-                sorting = true;
+                PauseSorting ();
                 foreach (StationSource child in StationSource.LoadAll (this, Account.UserName)) {
-                    if (Connection.Subscriber ||
-                            (!child.Type.SubscribersOnly &&
-                                !(child.Type == StationType.Personal && child.Arg != null && child.Arg.Trim().ToLower() == last_username.Trim().ToLower())))
-                    {
-                        AddChildSource (child);
-                    }
+                    AddChildSource (child);
                 }
-                sorting = false;
+                ResumeSorting ();
                 SortChildSources ();
             }
         }
@@ -265,7 +209,7 @@ namespace Banshee.Lastfm.Radio
         }
 
         public override bool HasProperties {
-            get { return true; }
+            get { return false; }
         }
 
         private void HandleConnectionStateChanged (object sender, ConnectionStateChangedArgs args)
@@ -273,20 +217,16 @@ namespace Banshee.Lastfm.Radio
             UpdateUI ();
         }
 
-        /*private void HandleChildUpdated (object sender, EventArgs args)
-        {
-            SortChildSources ();
-        }*/
-
         private void UpdateUI ()
         {
             bool have_user = Account.UserName != null;
-            bool have_pass = Account.CryptedPassword != null;
-            
-            if (have_pass) {
-                LastPassSchema.Set (Account.CryptedPassword);
+            bool have_session_key = Account.SessionKey != null;
+
+            if (have_session_key) {
+                LastSessionKeySchema.Set (Account.SessionKey);
+                LastIsSubscriberSchema.Set (Account.Subscriber);
             }
-            
+
             if (have_user) {
                 SetUserName (Account.UserName);
             } else {
@@ -317,29 +257,128 @@ namespace Banshee.Lastfm.Radio
         internal static void SetStatus (SourceMessage status_message, LastfmSource lastfm, bool error, ConnectionState state)
         {
             status_message.FreezeNotify ();
-            if (error && (state == ConnectionState.NoAccount || state == ConnectionState.InvalidAccount)) {
-                status_message.AddAction (new MessageAction (Catalog.GetString ("Account Settings"),
-                    delegate { lastfm.Actions.ShowLoginDialog (); }));
-                status_message.AddAction (new MessageAction (Catalog.GetString ("Join Last.fm"),
-                    delegate { lastfm.Account.SignUp (); }));
+            if (error) {
+                if (state == ConnectionState.NoAccount || state == ConnectionState.InvalidAccount || state == ConnectionState.NotAuthorized) {
+                    status_message.AddAction (new MessageAction (Catalog.GetString ("Account Settings"),
+                        delegate { lastfm.Actions.ShowLoginDialog (); }));
+                }
+                if (state == ConnectionState.NoAccount || state == ConnectionState.InvalidAccount) {
+                    status_message.AddAction (new MessageAction (Catalog.GetString ("Join Last.fm"),
+                        delegate { lastfm.Account.SignUp (); }));
+                }
             }
             status_message.ThawNotify ();
         }
 
+        private SourcePage pref_page;
+        private Section pref_section;
+        private SchemaPreference<string> user_pref;
+
+        private void InstallPreferences ()
+        {
+            PreferenceService service = ServiceManager.Get<PreferenceService> ();
+            if (service == null) {
+                return;
+            }
+
+            service.InstallWidgetAdapters += OnPreferencesServiceInstallWidgetAdapters;
+            pref_page = new Banshee.Preferences.SourcePage (this);
+            pref_section = pref_page.Add (new Section ("lastfm-account", Catalog.GetString ("Account"), 20));
+            pref_section.ShowLabel = false;
+
+            user_pref = new SchemaPreference<string> (LastUserSchema, Catalog.GetString ("_Username"));
+            pref_section.Add (user_pref);
+            pref_section.Add (new VoidPreference ("lastfm-signup"));
+        }
+
+        private void UninstallPreferences ()
+        {
+            PreferenceService service = ServiceManager.Get<PreferenceService> ();
+            if (service == null || pref_page == null) {
+                return;
+            }
+
+            service.InstallWidgetAdapters -= OnPreferencesServiceInstallWidgetAdapters;
+            pref_page.Dispose ();
+            pref_page = null;
+        }
+
+        private bool Authorized { get { return !String.IsNullOrEmpty (account.SessionKey); } }
+        private bool NeedAuth { get { return !String.IsNullOrEmpty (user_pref.Value) && !Authorized; } }
+
+        private void OnPreferencesServiceInstallWidgetAdapters (object sender, EventArgs args)
+        {
+            if (pref_section == null) {
+                return;
+            }
+
+            var user_entry = new Gtk.Entry (user_pref.Value ?? "");
+            user_entry.Changed += (o, a) => { user_pref.Value = user_entry.Text; };
+
+            var auth_button = new Gtk.Button (Authorized ? Catalog.GetString ("Authorized!") : Catalog.GetString ("Authorize..."));
+            user_pref.ValueChanged += (s) => { auth_button.Sensitive = NeedAuth; };
+            auth_button.Sensitive = NeedAuth;
+            auth_button.TooltipText = Catalog.GetString ("Open Last.fm in a browser, giving you the option to authorize Banshee to work with your account");
+            auth_button.Clicked += delegate {
+                account.SessionKey = null;
+                account.RequestAuthorization ();
+            };
+
+            var signup_button = new Gtk.LinkButton (account.SignUpUrl, Catalog.GetString ("Sign up for Last.fm"));
+            signup_button.Xalign = 0f;
+
+            var refresh_button = new Gtk.Button (new Gtk.Image (Gtk.Stock.Refresh, Gtk.IconSize.Button));
+            user_pref.ValueChanged += (s) => { refresh_button.Sensitive = NeedAuth; };
+            refresh_button.Sensitive = NeedAuth;
+            refresh_button.TooltipText = Catalog.GetString ("Check if Banshee has been authorized");
+            refresh_button.Clicked += delegate {
+                if (String.IsNullOrEmpty (account.UserName) || account.SessionKey == null) {
+                    account.UserName = LastUserSchema.Get ();
+                    account.FetchSessionKey ();
+                    account.Save ();
+                    LastSessionKeySchema.Set (account.SessionKey);
+                }
+
+                auth_button.Sensitive = refresh_button.Sensitive = NeedAuth;
+                auth_button.Label = Authorized
+                    ? Catalog.GetString ("Authorized!")
+                    : Catalog.GetString ("Authorize...");
+            };
+
+            var auth_box = new Gtk.HBox () { Spacing = 6 };
+            auth_box.PackStart (user_entry, true, true, 0);
+            auth_box.PackStart (auth_button, false, false, 0);
+            auth_box.PackStart (refresh_button, false, false, 0);
+            auth_box.ShowAll ();
+
+            signup_button.Visible = String.IsNullOrEmpty (user_pref.Value);
+
+            var button_box = new Gtk.HBox () { Spacing = 6 };
+            button_box.PackStart (new Badge (account) { Visible = true}, false, false, 0);
+            button_box.PackStart (signup_button, true, true, 0);
+
+            user_pref.DisplayWidget = auth_box;
+            pref_section["lastfm-signup"].DisplayWidget = button_box;
+        }
+
+        public override string PreferencesPageId {
+            get { return pref_page.Id; }
+        }
+
         public static readonly SchemaEntry<bool> EnabledSchema = new SchemaEntry<bool> (
             "plugins.lastfm", "enabled", false, "Extension enabled", "Last.fm extension enabled"
         );
 
-        public static readonly SchemaEntry<int> StationSortSchema = new SchemaEntry<int> (
-            "plugins.lastfm", "station_sort", 0, "Station sort criteria", "Last.fm station sort criteria. 0 = name, 1 = play count, 2 = type"
-        );
-
         public static readonly SchemaEntry<string> LastUserSchema = new SchemaEntry<string> (
             "plugins.lastfm", "username", "", "Last.fm user", "Last.fm username"
         );
 
-        public static readonly SchemaEntry<string> LastPassSchema = new SchemaEntry<string> (
-            "plugins.lastfm", "password_hash", "", "Last.fm password", "Last.fm password (hashed)"
+        public static readonly SchemaEntry<string> LastSessionKeySchema = new SchemaEntry<string> (
+            "plugins.lastfm", "session_key", "", "Last.fm session key", "Last.fm session key used in authenticated calls"
+        );
+
+        public static readonly SchemaEntry<bool> LastIsSubscriberSchema = new SchemaEntry<bool> (
+            "plugins.lastfm", "subscriber", false, "User is Last.fm subscriber", "User is Last.fm subscriber"
         );
 
         public static readonly SchemaEntry<bool> ExpandedSchema = new SchemaEntry<bool> (
