@@ -56,65 +56,62 @@ namespace Banshee.Podcasting.Gui
     public class PodcastActions : BansheeActionGroup
     {
         private uint actions_id;
-        private PodcastSource source;
-        
+        private DatabaseSource last_source;
+
         public PodcastActions (PodcastSource source) : base (ServiceManager.Get<InterfaceActionService> (), "Podcast")
         {
-            this.source = source;
-
-            AddImportant (new ActionEntry[] {
+            AddImportant (
                 new ActionEntry (
                     "PodcastUpdateAllAction", Stock.Refresh,
-                     Catalog.GetString ("Update Podcasts"), null,//"<control><shift>U",
-                     Catalog.GetString ("Refresh All Podcasts"), 
+                     Catalog.GetString ("Check for New Episodes"), null,//"<control><shift>U",
+                     Catalog.GetString ("Refresh All Podcasts"),
                      OnPodcastUpdateAll
                 ),
                 new ActionEntry (
                     "PodcastAddAction", Stock.Add,
-                     Catalog.GetString ("Subscribe to Podcast"),"<control><shift>F", 
+                     Catalog.GetString ("Subscribe to Podcast..."),"<control><shift>F",
                      Catalog.GetString ("Subscribe to a new podcast"),
                      OnPodcastAdd
-                )         
-            });
-            
-            Add (new ActionEntry [] {
-                new ActionEntry("PodcastFeedPopupAction", null, 
+                )
+            );
+
+            Add (
+                new ActionEntry("PodcastFeedPopupAction", null,
                     String.Empty, null, null, OnFeedPopup),
-                    
+
                 new ActionEntry (
                     "PodcastDeleteAction", Stock.Delete,
                      Catalog.GetString ("Unsubscribe and Delete"),
-                     null, String.Empty, 
+                     null, String.Empty,
                      OnPodcastDelete
                 ),
                 new ActionEntry (
                     "PodcastUpdateFeedAction", Stock.Refresh,
-                     /* Translators: this is a verb used as a button name, not a noun*/
                      Catalog.GetString ("Check for New Episodes"),
-                     null, String.Empty, 
+                     null, String.Empty,
                      OnPodcastUpdate
                 ),
                 new ActionEntry (
                     "PodcastDownloadAllAction", Stock.Save,
                      Catalog.GetString ("Download All Episodes"),
-                     null, String.Empty, 
+                     null, String.Empty,
                      OnPodcastDownloadAllEpisodes
                 ),
                 new ActionEntry (
                     "PodcastHomepageAction", Stock.JumpTo,
                      Catalog.GetString ("Visit Podcast Homepage"),
-                     null, String.Empty, 
+                     null, String.Empty,
                      OnPodcastHomepage
                 ),
                 new ActionEntry (
                     "PodcastPropertiesAction", Stock.Properties,
                      Catalog.GetString ("Properties"),
-                     null, String.Empty, 
+                     null, String.Empty,
                      OnPodcastProperties
                 ),
                 new ActionEntry (
                     "PodcastItemMarkNewAction", null,
-                     Catalog.GetString ("Mark as New"), 
+                     Catalog.GetString ("Mark as New"),
                      null, String.Empty,
                      OnPodcastItemMarkNew
                 ),
@@ -127,45 +124,42 @@ namespace Banshee.Podcasting.Gui
                     "PodcastItemDownloadAction", Stock.Save,
                      /* Translators: this is a verb used as a button name, not a noun*/
                      Catalog.GetString ("Download Podcast(s)"),
-                     "<control><shift>D", String.Empty, 
+                     "<control><shift>D", String.Empty,
                      OnPodcastItemDownload
                 ),
                 new ActionEntry (
                     "PodcastItemCancelAction", Stock.Cancel,
                      Catalog.GetString ("Cancel Download"),
-                     "<control><shift>C", String.Empty, 
+                     "<control><shift>C", String.Empty,
                      OnPodcastItemCancel
                 ),
                 new ActionEntry (
                     "PodcastItemDeleteFileAction", Stock.Remove,
                      Catalog.GetString ("Remove Downloaded File(s)"),
-                     null, String.Empty, 
+                     null, String.Empty,
                      OnPodcastItemDeleteFile
                 ),
                 new ActionEntry (
                     "PodcastItemLinkAction", Stock.JumpTo,
                      Catalog.GetString ("Visit Website"),
-                     null, String.Empty, 
+                     null, String.Empty,
                      OnPodcastItemLink
                 ),
                 new ActionEntry (
                     "PodcastItemPropertiesAction", Stock.Properties,
                      Catalog.GetString ("Properties"),
-                     null, String.Empty, 
+                     null, String.Empty,
                      OnPodcastItemProperties
                 )
-            });
-            
+            );
+
+            this["PodcastAddAction"].ShortLabel = Catalog.GetString ("Subscribe to Podcast");
+
             actions_id = Actions.UIManager.AddUiFromResource ("GlobalUI.xml");
             Actions.AddActionGroup (this);
 
             ServiceManager.SourceManager.ActiveSourceChanged += HandleActiveSourceChanged;
-            
-            source.TrackModel.Selection.Changed += delegate { Banshee.Base.ThreadAssist.ProxyToMain (UpdateItemActions); };
-            source.FeedModel.Selection.Changed += delegate { Banshee.Base.ThreadAssist.ProxyToMain (UpdateFeedActions); };
-            
-            UpdateFeedActions ();
-            UpdateItemActions ();
+            OnSelectionChanged (null, null);
         }
 
         public override void Dispose ()
@@ -178,6 +172,31 @@ namespace Banshee.Podcasting.Gui
 #region State Event Handlers
 
         private void HandleActiveSourceChanged (SourceEventArgs args)
+        {
+            if (last_source != null) {
+                foreach (IFilterListModel filter in last_source.AvailableFilters) {
+                    filter.Selection.Changed -= OnSelectionChanged;
+                }
+                last_source.TrackModel.Selection.Changed -= OnSelectionChanged;
+                last_source = null;
+            }
+
+            last_source = args.Source as DatabaseSource;
+            if (IsPodcastSource) {
+                if (last_source != null) {
+                    foreach (IFilterListModel filter in last_source.AvailableFilters) {
+                        filter.Selection.Changed += OnSelectionChanged;
+                    }
+                    last_source.TrackModel.Selection.Changed += OnSelectionChanged;
+                }
+            } else {
+                last_source = null;
+            }
+
+            OnSelectionChanged (null, null);
+        }
+
+        private void OnSelectionChanged (object o, EventArgs args)
         {
             Banshee.Base.ThreadAssist.ProxyToMain (delegate {
                 UpdateFeedActions ();
@@ -189,29 +208,62 @@ namespace Banshee.Podcasting.Gui
 
 #region Utility Methods
 
+        private DatabaseSource ActiveDbSource {
+            get { return last_source; }
+        }
+
+        private bool IsPodcastSource {
+            get {
+                return ActiveDbSource != null && (ActiveDbSource is PodcastSource || ActiveDbSource.Parent is PodcastSource);
+            }
+        }
+
+        public PodcastFeedModel ActiveFeedModel {
+            get {
+                if (ActiveDbSource == null) {
+                    return null;
+                } else if (ActiveDbSource is PodcastSource) {
+                    return (ActiveDbSource as PodcastSource).FeedModel;
+                } else {
+                    foreach (IFilterListModel filter in ActiveDbSource.AvailableFilters) {
+                        if (filter is PodcastFeedModel) {
+                            return filter as PodcastFeedModel;
+                        }
+                    }
+                    return null;
+                }
+            }
+        }
+
         private void UpdateItemActions ()
         {
-            if (ServiceManager.SourceManager.ActiveSource == source) {
-                bool has_single_selection = source.TrackModel.Selection.Count == 1;
+            if (IsPodcastSource) {
+                bool has_single_selection = ActiveDbSource.TrackModel.Selection.Count == 1;
                 UpdateActions (true, has_single_selection,
                    "PodcastItemLinkAction"
                 );
             }
         }
-        
+
         private void UpdateFeedActions ()
         {
-            if (ServiceManager.SourceManager.ActiveSource == source) {
-                bool has_single_selection = source.FeedModel.Selection.Count == 1;
-                bool all_selected = source.FeedModel.Selection.AllSelected;
+            if (IsPodcastSource) {
+                bool has_single_selection = ActiveFeedModel.Selection.Count == 1;
+                bool all_selected = ActiveFeedModel.Selection.AllSelected;
 
                 UpdateActions (true, has_single_selection && !all_selected,
-                    "PodcastDeleteAction", "PodcastUpdateFeedAction", "PodcastHomepageAction",
+                    "PodcastHomepageAction",
                     "PodcastPropertiesAction"
                 );
+
+                UpdateActions (true, ActiveFeedModel.Selection.Count > 0 && !all_selected,
+                    "PodcastDeleteAction",
+                    "PodcastUpdateFeedAction",
+                    "PodcastDownloadAllAction"
+                );
             }
         }
-        
+
         private void SubscribeToPodcast (Uri uri, FeedAutoDownload syncPreference)
         {
             FeedsManager.Instance.FeedManager.CreateFeed (uri.ToString (), syncPreference);
@@ -219,40 +271,42 @@ namespace Banshee.Podcasting.Gui
 
         private IEnumerable<TrackInfo> GetSelectedItems ()
         {
-            return new List<TrackInfo> (source.TrackModel.SelectedItems);
+            return new List<TrackInfo> (ActiveDbSource.TrackModel.SelectedItems);
         }
 
 #endregion
-            
+
+
 #region Action Handlers
 
         private void OnFeedPopup (object o, EventArgs args)
         {
-            if (source.FeedModel.Selection.AllSelected)
+            if (ActiveFeedModel.Selection.AllSelected) {
                 ShowContextMenu ("/PodcastAllFeedsContextMenu");
-            else
+            } else {
                 ShowContextMenu ("/PodcastFeedPopup");
+            }
         }
 
         private void RunSubscribeDialog ()
-        {        
+        {
             Uri feedUri = null;
             FeedAutoDownload syncPreference;
-            
+
             PodcastSubscribeDialog subscribeDialog = new PodcastSubscribeDialog ();
-            
+
             ResponseType response = (ResponseType) subscribeDialog.Run ();
             syncPreference = subscribeDialog.SyncPreference;
-            
+
             subscribeDialog.Destroy ();
 
             if (response == ResponseType.Ok) {
                 string url = subscribeDialog.Url.Trim ().Trim ('/');
-                
+
                 if (String.IsNullOrEmpty (subscribeDialog.Url)) {
                     return;
                 }
-                
+
 				if (!TryParseUrl (url, out feedUri)) {
                     HigMessageDialog.RunHigMessageDialog (
                         null,
@@ -263,19 +317,19 @@ namespace Banshee.Podcasting.Gui
                         Catalog.GetString ("Podcast URL is invalid.")
                     );
 				} else {
-				    SubscribeToPodcast (feedUri, syncPreference); 
+				    SubscribeToPodcast (feedUri, syncPreference);
 				}
-            }        
+            }
         }
-        
-        /*private void RunConfirmDeleteDialog (bool feed, 
-                                             int selCount, 
-                                             out bool delete, 
+
+        /*private void RunConfirmDeleteDialog (bool feed,
+                                             int selCount,
+                                             out bool delete,
                                              out bool deleteFiles)
         {
-            
+
             delete = false;
-            deleteFiles = false;        
+            deleteFiles = false;
             string header = null;
             int plural = (feed | (selCount > 1)) ? 2 : 1;
 
@@ -284,27 +338,27 @@ namespace Banshee.Podcasting.Gui
             } else {
                 header = Catalog.GetPluralString ("Delete episode?", "Delete episodes?", selCount);
             }
-                
+
             HigMessageDialog md = new HigMessageDialog (
                 ServiceManager.Get<GtkElementsService> ("GtkElementsService").PrimaryWindow,
-                DialogFlags.DestroyWithParent, 
+                DialogFlags.DestroyWithParent,
                 MessageType.Question,
-                ButtonsType.None, header, 
+                ButtonsType.None, header,
                 Catalog.GetPluralString (
                     "Would you like to delete the associated file?",
-                    "Would you like to delete the associated files?", plural                
+                    "Would you like to delete the associated files?", plural
                 )
             );
-            
+
             md.AddButton (Stock.Cancel, ResponseType.Cancel, true);
             md.AddButton (
                 Catalog.GetPluralString (
                     "Keep File", "Keep Files", plural
                 ), ResponseType.No, false
             );
-            
+
             md.AddButton (Stock.Delete, ResponseType.Yes, false);
-            
+
             try {
                 switch ((ResponseType)md.Run ()) {
                 case ResponseType.Yes:
@@ -313,12 +367,12 @@ namespace Banshee.Podcasting.Gui
                 case ResponseType.No:
                     delete = true;
                     break;
-                }                
+                }
             } finally {
                 md.Destroy ();
-            }       
+            }
         }*/
-        
+
 		private bool TryParseUrl (string url, out Uri uri)
 		{
 			uri = null;
@@ -327,35 +381,35 @@ namespace Banshee.Podcasting.Gui
             try {
                 uri = new Uri (url);
 				
-				if (uri.Scheme == Uri.UriSchemeHttp || 
+				if (uri.Scheme == Uri.UriSchemeHttp ||
 				    uri.Scheme == Uri.UriSchemeHttps) {
 					ret = true;
 				}
             } catch {}
-            
+
             return ret;			
 		}
 
         /*private void OnFeedSelectionChangedHandler (object sender, EventArgs e)
         {
             lock (sync) {
-                if (!disposed || disposing) {              
+                if (!disposed || disposing) {
                     if (source.FeedModel.SelectedItems.Count == 0) {
                         source.FeedModel.Selection.Select (0);
                     }
-                    
+
                     if (source.FeedModel.Selection.Contains (0)) {
                         itemModel.FilterOnFeed (Feed.All);
                     } else {
                         itemModel.FilterOnFeeds (source.FeedModel.CopySelectedItems ());
                     }
-                    
+
                     itemModel.Selection.Clear ();
                 }
             }
         }
-        
-        private void OnPodcastItemRowActivatedHandler (object sender, 
+
+        private void OnPodcastItemRowActivatedHandler (object sender,
                                                        RowActivatedArgs<PodcastItem> e)
         {
             lock (sync) {
@@ -372,145 +426,146 @@ namespace Banshee.Podcasting.Gui
         {
             RunSubscribeDialog ();
         }
-        
+
         private void OnPodcastUpdate (object sender, EventArgs e)
         {
-            foreach (Feed feed in source.FeedModel.SelectedItems) {
+            foreach (Feed feed in ActiveFeedModel.SelectedItems) {
                 feed.Update ();
             }
-        }        
-        
+        }
+
         private void OnPodcastUpdateAll (object sender, EventArgs e)
         {
             foreach (Feed feed in Feed.Provider.FetchAll ()) {
                 feed.Update ();
             }
-        }      
-        
+        }
+
         private void OnPodcastDelete (object sender, EventArgs e)
         {
-            Feed feed = source.FeedModel.FocusedItem;
-            if (feed != null) {
-                feed.Delete (true);
-            }
-        }
-        
-        private void OnPodcastDownloadAllEpisodes (object sender, EventArgs e)
-        {
-            Feed feed = source.FeedModel.FocusedItem;
-            if (feed != null) {
-                foreach (FeedItem item in feed.Items) {
-                    item.Enclosure.AsyncDownload ();
+            foreach (Feed feed in ActiveFeedModel.SelectedItems) {
+                if (feed != null) {
+                    feed.Delete (true);
                 }
             }
-        }   
+        }
+
+        private void OnPodcastDownloadAllEpisodes (object sender, EventArgs e)
+        {
+            foreach (Feed feed in ActiveFeedModel.SelectedItems) {
+                if (feed != null) {
+                    foreach (FeedItem item in feed.Items) {
+                        item.Enclosure.AsyncDownload ();
+                    }
+                }
+            }
+        }
 
         private void OnPodcastItemDeleteFile (object sender, EventArgs e)
         {
-            foreach (PodcastTrackInfo pi in GetSelectedItems ()) {
+            foreach (PodcastTrackInfo pi in PodcastTrackInfo.From (GetSelectedItems ())) {
                 if (pi.Enclosure.LocalPath != null)
                     pi.Enclosure.DeleteFile ();
             }
-        }  
+        }
 
         private void OnPodcastHomepage (object sender, EventArgs e)
         {
-            Feed feed = source.FeedModel.FocusedItem;
+            Feed feed = ActiveFeedModel.FocusedItem;
             if (feed != null && !String.IsNullOrEmpty (feed.Link)) {
                 Banshee.Web.Browser.Open (feed.Link);
-            }   
-        }   
+            }
+        }
 
         private void OnPodcastProperties (object sender, EventArgs e)
         {
-            Feed feed = source.FeedModel.FocusedItem;
+            Feed feed = ActiveFeedModel.FocusedItem;
             if (feed != null) {
                 new PodcastFeedPropertiesDialog (feed).Run ();
             }
-        }  
+        }
 
         private void OnPodcastItemProperties (object sender, EventArgs e)
         {
                 /*ReadOnlyCollection<PodcastItem> items = itemModel.CopySelectedItems ();
-                
+
                 if (items != null && items.Count == 1) {
                     new PodcastPropertiesDialog (items[0]).Run ();
-                } */                
-        } 
+                } */
+        }
 
         private void OnPodcastItemMarkNew (object sender, EventArgs e)
         {
             MarkPodcastItemSelection (false);
         }
-        
+
         private void OnPodcastItemMarkOld (object sender, EventArgs e)
         {
             MarkPodcastItemSelection (true);
-        }     
-        
-        private void MarkPodcastItemSelection (bool markRead) 
+        }
+
+        private void MarkPodcastItemSelection (bool markRead)
         {
-            TrackInfo new_selection_track = source.TrackModel [source.TrackModel.Selection.LastIndex + 1];
-            
+            TrackInfo new_selection_track = ActiveDbSource.TrackModel [ActiveDbSource.TrackModel.Selection.LastIndex + 1];
+
             PodcastService.IgnoreItemChanges = true;
-            
+
             bool any = false;
-            foreach (PodcastTrackInfo track in GetSelectedItems ()) {
+            foreach (PodcastTrackInfo track in PodcastTrackInfo.From (GetSelectedItems ())) {
                 if (track.Item.IsRead != markRead) {
                     track.Item.IsRead = markRead;
                     track.Item.Save ();
 
-                    if (track.Item.IsRead ^ track.PlayCount > 0) {
-                        track.PlayCount = track.Item.IsRead ? 1 : 0;
-                        track.Save (false);
+                    if (track.Item.IsRead ^ track.Track.PlayCount > 0) {
+                        track.Track.PlayCount = track.Item.IsRead ? 1 : 0;
+                        track.Track.Save (false);
                     }
                     any = true;
                 }
             }
-            
+
             PodcastService.IgnoreItemChanges = false;
-            
+
             if (any) {
-                source.Reload ();
-                
+                ActiveDbSource.Reload ();
+
                 // If we just removed all of the selected items from our view, we should select the
                 // item after the last removed item
-                if (source.TrackModel.Selection.Count == 0 && new_selection_track != null) {
-                    int new_i = source.TrackModel.IndexOf (new_selection_track);
+                if (ActiveDbSource.TrackModel.Selection.Count == 0 && new_selection_track != null) {
+                    int new_i = ActiveDbSource.TrackModel.IndexOf (new_selection_track);
                     if (new_i != -1) {
-                        source.TrackModel.Selection.Clear (false);
-                        source.TrackModel.Selection.FocusedIndex = new_i;
-                        source.TrackModel.Selection.Select (new_i);
+                        ActiveDbSource.TrackModel.Selection.Clear (false);
+                        ActiveDbSource.TrackModel.Selection.FocusedIndex = new_i;
+                        ActiveDbSource.TrackModel.Selection.Select (new_i);
                     }
                 }
             }
         }
-        
+
         private void OnPodcastItemCancel (object sender, EventArgs e)
         {
             /*
-                if (!disposed || disposing) {                    
+                if (!disposed || disposing) {
                     ReadOnlyCollection<PodcastItem> items = itemModel.CopySelectedItems ();
 
                     if (items != null) {
                         foreach (PodcastItem pi in items) {
                             pi.Enclosure.CancelAsyncDownload ();
                         }
-                    }                
+                    }
                 }*/
-        }        
-        
+        }
+
         private void OnPodcastItemDownload (object sender, EventArgs e)
         {
-            foreach (PodcastTrackInfo pi in GetSelectedItems ()) {
-                if (pi.Enclosure.DownloadStatus != FeedDownloadStatus.Downloaded)
-                    pi.Enclosure.AsyncDownload ();
+            foreach (PodcastTrackInfo pi in PodcastTrackInfo.From (GetSelectedItems ())) {
+                pi.Enclosure.AsyncDownload ();
             }
         }
-        
+
         private void OnPodcastItemLink (object sender, EventArgs e)
         {
-            PodcastTrackInfo track = source.TrackModel.FocusedItem as PodcastTrackInfo;
+            PodcastTrackInfo track = PodcastTrackInfo.From (ActiveDbSource.TrackModel.FocusedItem);
             if (track != null && !String.IsNullOrEmpty (track.Item.Link)) {
                 Banshee.Web.Browser.Open (track.Item.Link);
             }
